﻿using Microsoft.Win32;
using Syncfusion.DocIO.DLS;
using Syncfusion.Windows.Controls.RichTextBoxAdv;
using Syncfusion.Windows.Tools.Controls;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace Sample
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        #region Constructor
        /// <summary>
        /// Initializes a new instance of the <see cref="T:Sample.MainWindow">MainWindow</see> class. 
        /// </summary>
        /// <remarks></remarks>
        public MainWindow()
        {
            InitializeComponent();
        }
        #endregion
    }
    /// <summary>
    /// Represents the extension class for SfRichTextBoxAdv.
    /// </summary>
    public class SfRichTextBoxAdvExtension : SfRichTextBoxAdv
    {
        #region Fields
        bool skipUpdating = false;
        #endregion

        #region Properties
        /// <summary>
        /// Gets or Sets the text.
        /// </summary>
        public string Text
        {
            get
            {
                return (string)GetValue(TextProperty);
            }
            set
            {
                SetValue(TextProperty, value);
            }
        }
        #endregion

        #region Constructor
        /// <summary>
        /// Initializes the instance of SfRichTextBoxAdvExtension class.
        /// </summary>
        public SfRichTextBoxAdvExtension()
        {
            // Wires the ContentChanged event.
            this.ContentChanged += RTE_ContentChanged;
        }
        #endregion

        #region Static Dependency Properties
        /// <summary>
        /// Using as a backing store for Text dependency property to enable styling, animation etc.
        /// </summary>
        public static readonly DependencyProperty TextProperty = DependencyProperty.Register("Text", typeof(string), typeof(SfRichTextBoxAdvExtension), new PropertyMetadata(string.Empty, new PropertyChangedCallback(OnTextChanged)));
        #endregion

        #region Static Events
        /// <summary>
        /// Called when text changed.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="e"></param>
        private static void OnTextChanged(DependencyObject obj, DependencyPropertyChangedEventArgs e)
        {
            SfRichTextBoxAdvExtension richTextBox = (SfRichTextBoxAdvExtension)obj;
            //Update the document with the text.
            richTextBox.UpdateDocument((string)e.NewValue);
        }
        #endregion

        #region Events
        /// <summary>
        /// Called when content changes in SfRichTextBoxAdv.
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="args"></param>
        void RTE_ContentChanged(object obj, ContentChangedEventArgs args)
        {
            if (this.Document != null)
            {
                // To skip internal updation of document on setting Text property.
                skipUpdating = true;
                Stream stream = new MemoryStream();
                // Saves the document as text Stream.
                this.Save(stream, FormatType.Txt);
                stream.Position = 0;
                // Reads the stream and assigned the string to Text property
                using (StreamReader reader = new StreamReader(stream))
                {
                    this.Text = reader.ReadToEnd();
                }
                skipUpdating = false;
            }
        }
        #endregion

        #region Implementation
        /// <summary>
        /// Updates the document.
        /// </summary>
        /// <param name="text">The text.</param>
        private void UpdateDocument(string text)
        {
            // If text property is set internally means, skip updating the document.
            if (!skipUpdating && !string.IsNullOrEmpty(text))
            {
                Stream stream = new MemoryStream();
                // Convert the text string to byte array.
                byte[] bytes = Encoding.UTF8.GetBytes(text);
                // Writes the byte array to stream.
                stream.Write(bytes, 0, bytes.Length);
                stream.Position = 0;
                //Load the Text stream.
                Load(stream, FormatType.Txt);
            }
        }
        /// <summary>
        /// Disposes the instance.
        /// </summary>
        public void Dispose()
        {
            this.ContentChanged -= RTE_ContentChanged;
            ClearValue(TextProperty);
            base.Dispose();
        }
        #endregion
    }

    /// <summary>
    /// Implements a type converter for converting <see cref="double">Double</see> to and from <see cref="string">String</see> type.
    /// </summary>
    public class FontSizeDoubleStringConverter : IValueConverter
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="DoubleStringConverter"/> class.
        /// </summary>
        public FontSizeDoubleStringConverter()
        {

        }

        /// <summary>
        /// Converts the <see cref="double">Double</see> value to the respective <see cref="string">String</see> value.
        /// </summary>
        /// <param name="value">The source data being passed to the target.</param>
        /// <param name="targetType"> The type of the target property.</param>
        /// <param name="parameter">An optional parameter to be used in the converter logic.</param>
        /// <param name="language">The language of the conversion.</param>
        /// <returns>
        /// If the <paramref name="value"/> is greater than zero, returns respective <see cref="string"/>.
        /// By default empty string is returned.
        /// </returns>
        /// <remarks>Modifies the source data before passing it to the target for display in the UI.</remarks>
        public object Convert(object value, Type targetType, object parameter, System.Globalization.CultureInfo language)
        {
            double factor = (double)value;
            factor = (factor * 72) / 96;
            if (factor > 0)
            {
                if (factor % 1 == 0)
                    return Math.Round(factor, 0).ToString();
                return Math.Round(factor, 1).ToString();
            }
            return " ";
        }
        /// <summary>
        /// Converts the <see cref="string">String</see> value to the respective <see cref="double">Double</see> value.
        /// </summary>
        /// <param name="value">The target data being passed to the source.</param>
        /// <param name="targetType">The type of the target property, specified by a helper structure that wraps the type name.</param>
        /// <param name="parameter"> An optional parameter to be used in the converter logic.</param>
        /// <param name="language">The language of the conversion.</param>
        /// <returns>
        /// Returns the <see cref="double"/> value for the specified <paramref name="value"/>.
        /// </returns>
        /// <remarks>
        /// Modifies the target data before passing it to the source object. This method is called only in TwoWay bindings.
        /// </remarks>
        public object ConvertBack(object value, Type targetType, object parameter, System.Globalization.CultureInfo language)
        {
            double size = 0d;
            if (value != null)
                double.TryParse(value.ToString(), out size);
            return (size * 96) / 72;
        }
    }
}
